
/**
 * @imports
 */
import _intersect from '@web-native-js/commons/arr/intersect.js';
import _with from '@web-native-js/commons/obj/with.js';
import getOffsets from './getOffsets.js.js.js.js';
import Destination from './Destination.js';

/**
 * Offsets an element to a reference point in the UI.
 */
export default class Offset extends Destination {

    /**
     * @inheritdoc
     */
    constructor(el, reference, options = {}) {
        super(...arguments);
        var start = {x:'left', y:'top'};
        var end = {x:'right', y:'bottom'};
        var offsets = _intersect(['x', 'y'], Object.keys(this.coords)).reduce((obj, axis) => {
            switch(this.alignment[axis].keyword) {
                case 'before':
                    return this.options.alternateAnchors ? _with(obj, start[axis], this.coords[axis]) : _with(obj, end[axis], - this.coords[axis]);
                case 'after':
                    return this.options.alternateAnchors ? _with(obj, end[axis], - this.coords[axis]) : _with(obj, start[axis], this.coords[axis]);
                case 'end':
                    return _with(obj, end[axis], - this.coords[axis]);
                default: // center,start
                    return _with(obj, start[axis], this.coords[axis]);
            }
        }, {});
        // -----------
        this.offsetsFormatted = {from: {}, to: {}, current: {
                from: {}, to: {}, progress: {},
            },
        };
        // -----------
        var currentOffsets = getOffsets(this.el, Object.keys(offsets));
        ['left', 'right', 'top', 'bottom'].forEach(name => {
            if (name in offsets) {
                this.offsetsFormatted.from[name] = 0;
                this.offsetsFormatted.to[name] = offsets[name] + currentOffsets[name];
                // ----------------
                this.offsetsFormatted.current.from[name] = currentOffsets[name];
                this.offsetsFormatted.current.to[name] = offsets[name];
                // ----------------
                this.offsetsFormatted.current.progress[name] = Math.abs(currentOffsets[name] / (offsets[name] + currentOffsets[name]));
            } else {
                this.offsetsFormatted.from[name] = 'auto';
                this.offsetsFormatted.to[name] = 'auto';
                // ----------------
                this.offsetsFormatted.current.from[name] = 'auto';
                this.offsetsFormatted.current.to[name] = 'auto';
            }
        });
    }
        
    /**
     * Plays the Translation.
     * 
     * @param object    timing
     * 
     * @return Promise
     */
    play(timing = {}) {
        return play(this.el, [this.translateFormatted.current.from, this.translateFormatted.to], timing);
    }

    /**
     * Sets the Translation by CSS.
     * 
     * @return Promise
     */
    css() {
        return cssAsync(this.el, {transform: this.translateFormatted.to});
    }
};
