
/**
 * @imports
 */
import Rect from './Rect.js';

/**
 * The Intersection class.
 */
export default class Intersection extends Rect {

    /**
     * Returns coordinates of the intersection between two rects.
     *
    * @param Rect|Element|Event|window 	rect1
    * @param Rect|Element|Event|window 	rect2
     *
     * @return Intersection
     */
    static of(rect1, rect2) {
        rect1 = Rect.of(rect1);
        rect2 = Rect.of(rect2);
        var $props = {
            left: rect1.left - rect2.left,
            top: rect1.top - rect2.top,
            right: (rect2.left + rect2.width) - (rect1.left + rect1.width),
            bottom: (rect2.top + rect2.height) - (rect1.top + rect1.height),
        };
        // More offsets
        var leftline = Math.max(rect1.left, rect2.left);
        var rightline = Math.min(rect1.left + rect1.width, rect2.left + rect2.width);
        var topline = Math.max(rect1.top, rect2.top);
        var bottomline = Math.min(rect1.top + rect1.height, rect2.top + rect2.height);
        $props.width = rightline > leftline ? rightline - leftline : 0;
        $props.height = bottomline > topline ? bottomline - topline : 0;
        // The raw values
        $props.rect1 = rect1;
        $props.rect2 = rect2;
        return new Intersection($props);
    }
};